
import wx
import os


class ImageViewer(wx.Frame):
    def __init__(self,parent,id,title='',**kw):
        wx.Frame.__init__(self,parent,id,title,**kw)

        self.buttonImageMapping = {}
        self.nameImageMapping = {}
        self.thumbnailHeight = 50.0
        
        # Build the user interface

        # - Sizer for laying out controls
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(self.sizer)

        # - Image panel for main image
        self.imagePanel = wx.Panel(self,-1)
        self.staticBitmap = wx.StaticBitmap(self.imagePanel,-1,style=wx.ALIGN_CENTRE)
        self.sizer.Add(self.imagePanel,1,wx.EXPAND)

        # - Thumbnail panel for smaller thumbnail images
        self.thumbnailSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.thumbnailPanel = wx.ScrolledWindow(self,-1,style=wx.HSCROLL|wx.BORDER_RAISED,size=(-1,100))
        self.thumbnailPanel.SetSizer(self.thumbnailSizer)
        self.thumbnailPanel.SetScrollbars(20,20,50,50)
        self.sizer.Add(self.thumbnailPanel,0,wx.EXPAND)

        # - "Watch Directory" button
        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        watchDirectoryLabel = wx.StaticText(self,-1,"Directory")
        self.watchDirectoryText = wx.TextCtrl(self,-1,style=wx.TE_PROCESS_ENTER,size=wx.Size(200,20))
        watchDirectoryButton = wx.Button(self,-1,'Browse...')
        hsizer.Add(watchDirectoryLabel,0)
        hsizer.Add(self.watchDirectoryText,1,wx.EXPAND)
        hsizer.Add(watchDirectoryButton,0)
        self.sizer.Add(hsizer,0,wx.EXPAND)
        wx.EVT_TEXT_ENTER(self, self.watchDirectoryText.GetId(), self.OnWatchDirectory)
        wx.EVT_BUTTON(self,watchDirectoryButton.GetId(),self.OnSelectDirectory)

        # - "Add Images" button
        addImagesButton = wx.Button(self,-1,'Add Images...')
        self.sizer.Add(addImagesButton)
        wx.EVT_BUTTON(self,addImagesButton.GetId(),self.OnAddImages)


        wx.EVT_SIZE(self,self.OnSize)
        wx.EVT_CLOSE(self,self.OnClose)

        self.currentImage = None
        self.watchDirectory = None

    def OpenImage(self,imageFile):
        '''
        Opens the specified image file
        '''

        # Create a bitmap from the incoming image
        imageFileName = os.path.basename(imageFile)
        image = wx.Image(imageFile,wx.BITMAP_TYPE_ANY)
        bitmap = wx.BitmapFromImage(image)
        
        # Create a panel for the thumbnail and image name
        bitmapsizer = wx.BoxSizer(wx.VERTICAL)
        bitmappanel = wx.Panel(self.thumbnailPanel,-1,style=wx.BORDER_RAISED)
        bitmappanel.SetSizer(bitmapsizer)
        
        # - create the thumbnail image
        scale = self.thumbnailHeight/image.GetHeight()
        scaledwidth = scale * image.GetWidth()
        scaledheight = self.thumbnailHeight
        scaledimage = image.Scale(scaledwidth,scaledheight)
        scaledbitmap = wx.BitmapFromImage(scaledimage)
        scaledbitmapctrl = wx.StaticBitmap(bitmappanel,-1,scaledbitmap)
        bitmapsizer.Add(scaledbitmapctrl,0,wx.ALIGN_CENTRE)

        # - create the image title control
        bitmaptitle = wx.StaticText(bitmappanel,-1,imageFileName)
        bitmapsizer.Add(bitmaptitle,0,wx.ALIGN_CENTRE)

        # Add the bitmap panel to the thumbnail panel
        self.thumbnailSizer.Add(bitmappanel,flag=wx.ALL,border=2)
        self.thumbnailSizer.Layout()

        # Force update the UI
        self.thumbnailPanel.Refresh()
        self.Refresh()

        # Track open images
        self.nameImageMapping[imageFileName] = image
        self.buttonImageMapping[scaledbitmapctrl] = image
        self.buttonImageMapping[bitmappanel] = image
        self.buttonImageMapping[bitmaptitle] = image

        # Add event handlers for mouse clicks on the bitmap panel and components
        wx.EVT_LEFT_DOWN(scaledbitmapctrl,self.OnImageSelect)
        wx.EVT_LEFT_DOWN(bitmappanel,self.OnImageSelect)
        wx.EVT_LEFT_DOWN(bitmaptitle,self.OnImageSelect)
        
    def ShowImage(self,image):
        '''
        Show the specified image
        '''
        # Show the selected image, scaling it to the current window size
        imagePanelSize = self.imagePanel.GetSize()
        imageSize = image.GetSize()
        imageAspectRatio =  imageSize[0] / float(imageSize[1])
        imagePanelAspectRatio = imagePanelSize[0] / float(imagePanelSize[1])
        if imagePanelAspectRatio > imageAspectRatio:
            newimagesize = (int(imageAspectRatio * imagePanelSize[1]), imagePanelSize[1])
        else:
            newimagesize = (imagePanelSize[0],int(imagePanelSize[0] / imageAspectRatio))
        windowscaledimage = image.Scale(newimagesize[0],newimagesize[1])
        windowscaledbitmap = wx.BitmapFromImage(windowscaledimage)
        self.staticBitmap.SetSize((newimagesize[0],newimagesize[1]))
        self.staticBitmap.SetBitmap(windowscaledbitmap)
        self.Refresh()
        self.currentImage = image


    #
    # UI Callbacks
    #
    def OnSelectDirectory(self,event=None):
        dialog = wx.DirDialog(self,style=wx.MULTIPLE)
        ret = dialog.ShowModal()
        if ret == wx.ID_OK:
            self.watchDirectoryText.SetValue( dialog.GetPath() )
            wx.CallAfter(self.OnWatchDirectory)
        dialog.Destroy()

    def OnAddImages(self,event=None):
        dialog = wx.FileDialog(self,style=wx.MULTIPLE)
        ret = dialog.ShowModal()
        if ret == wx.ID_OK:
            filenames = dialog.GetPaths()
            for f in filenames:
                wx.CallAfter(self.OpenImage,f)
        dialog.Destroy()

    def OnImageSelect(self,event):
        # Determine the image selected and show it
        o = event.GetEventObject()
        image = self.buttonImageMapping[o]
        self.ShowImage(image)

    def OnSize(self,event):
        if self.currentImage:
            self.ShowImage(self.currentImage)
        event.Skip()

    def OnClose(self,event):
        event.Skip()

    def OnWatchDirectory(self,event):
        event.Skip()


def main():
    wxapp = wx.PySimpleApp()
          
    wx.InitAllImageHandlers()

    # Create the image viewer and show it
    f = ImageViewer(None,-1,'Shared Image Viewer',size=(600,400))
    f.Show(1)

    # Open some images as an example
    imagelist = ['AGTk.png',
                 'AGimage.png',
                 'paraview_video.gif'
                 ]
    for image in imagelist:
        f.OpenImage(image)
    f.ShowImage(f.buttonImageMapping.values()[0])
    f.Refresh()
    f.sizer.Layout()

    # main loop
    wxapp.SetTopWindow(f)
    wxapp.MainLoop()




if __name__ == '__main__':
    main()
