

"""
SharedImageViewer.py

This is a sample Access Grid shared application.

This application is for coordinated viewing of images.  Users can select local
images for viewing, at which point they are uploaded to the Venue and other
users of the same SharedImageViewer application session automatically load and
view the image.
"""



import ImageViewer

from AccessGrid.SharedAppClient import SharedAppClient
from AccessGrid.Platform.Config import UserConfig
from AccessGrid.ClientProfile import ClientProfile
from AccessGrid.GUID import GUID
from AccessGrid.DataStoreClient import GetVenueDataStore, FileNotFound
from AccessGrid.Toolkit import Application
import os
import threading
import dirwatch

class SharedImageViewer(ImageViewer.ImageViewer):
    
    LOAD_IMAGE = "1"
    SHOW_IMAGE = "2"
    REMOVE_IMAGE = "3"

    imagelist = 'imagelist'
    
    def __init__( self, parent, id, title, appUrl, connectionId, **kw):
        ImageViewer.ImageViewer.__init__(self,parent,id,title,**kw)

        name = 'Shared Image Viewer'
    
        # Create shared application client
        self.sharedAppClient = SharedAppClient(name)
        self.log = self.sharedAppClient.InitLogging()
        self.tmpdir = UserConfig.instance().GetTempDir()

        # Get client profile
        try:
            clientProfileFile = os.path.join(UserConfig.instance().GetConfigDir(), "profile")
            self.clientProfile = ClientProfile(clientProfileFile)
        except:
            self.log.info("Could not load client profile, set clientProfile = None")
            self.clientProfile = None
    
        # Join the application session
        self.sharedAppClient.Join(appUrl, self.clientProfile)
        self.publicId = self.sharedAppClient.GetPublicId()
        self.venueURL = self.sharedAppClient.GetVenueURL()
        self.dataStoreClient = GetVenueDataStore(self.venueURL, connectionId)

        # Register event callbacks
        self.sharedAppClient.RegisterEventCallback(self.LOAD_IMAGE, self.HandleLoadImageEvent)
        self.sharedAppClient.RegisterEventCallback(self.SHOW_IMAGE, self.HandleShowImageEvent)


    def OnFilesChanged(self,changedFiles,removedFiles):
        print "OnFilesChanged:", changedFiles
        images = [f for f in changedFiles if f.split('.')[-1] in ['png','jpg','gif']]
        print "images = ", images
        wx.CallAfter(self.AddTheseImages,images)

    def DoLateJoin(self):
        '''
        Sync the local application with the shared application state
        (e.g., determine which images have been loaded, download and display them)
        '''
        keys = self.sharedAppClient.GetDataKeys()
        if SharedImageViewer.imagelist in keys:
            filenamesstr = self.sharedAppClient.GetData(SharedImageViewer.imagelist)
            filenames = eval(filenamesstr)
            successFilenames = []
            for f in filenames:
                try:
                    self.DownloadAndOpenImage(f)
                    successFilenames.append(f)
                except FileNotFound:
                    print 'File not found in Venue: ', f, '; removing'
            if len(successFilenames) < len(filenames):
                self.sharedAppClient.SetData(SharedImageViewer.imagelist,successFilenames)
                
    def Shutdown(self):
        '''
        Exit the application service
        '''
        self.sharedAppClient.Shutdown()
        os._exit(1)
        

    def DownloadAndOpenImage(self, filename):
        downloadfilename = os.path.join(self.tmpdir,filename)
        try:
            self.dataStoreClient.Download(filename,downloadfilename)
        except FileNotFound:
            self.dataStoreClient.LoadData()
            self.dataStoreClient.Download(filename,downloadfilename)
        self.OpenImage(downloadfilename)


    #
    # Event Channel callbacks
    #
    def HandleLoadImageEvent(self, event):
        filename = None
        try:
            if event.senderId != self.publicId:
                filename = event.GetData()
                self.DownloadAndOpenImage(filename)
        except:
            import traceback
            traceback.print_exc()
            self.log.exception('Error handling LoadImage event')
            wx.MessageDialog('Error loading file '+filename)

    def HandleShowImageEvent(self, event):
        try:
            filename = event.GetData()
            print 'GotShowImage event for image: ', filename
            if filename in self.nameImageMapping.keys():
                self.ShowImage(self.nameImageMapping[filename])
        except:
            import traceback
            traceback.print_exc()
            self.log.exception('Error handling ShowImage event')
            wx.MessageDialog('Error showing file '+filename)


    #
    # UI Callbacks
    #


    def AddTheseImages(self,filenames):
        for f in filenames:
            wx.CallAfter(self.OpenImage,f)

            # upload the file to venue
            self.dataStoreClient.Upload(f)

            # send event notifying others to upload
            filename = os.path.basename(f)
            self.sharedAppClient.SendEvent(SharedImageViewer.LOAD_IMAGE,filename)

        # update the application state
        keys= self.sharedAppClient.GetDataKeys()
        filelist = map( lambda x: os.path.basename(x), filenames)
        if SharedImageViewer.imagelist not in keys:
            self.sharedAppClient.SetData(SharedImageViewer.imagelist,filelist)
        else:
            myfileliststr = self.sharedAppClient.GetData(SharedImageViewer.imagelist)
            myfilelist = eval(myfileliststr)
            myfilelist += filelist
            self.sharedAppClient.SetData(SharedImageViewer.imagelist,myfilelist)


    def OnWatchDirectory(self,event=None):
        watchdir = self.watchDirectoryText.GetValue()
        print "watchdir = ", watchdir
        if watchdir:
            threading.Thread(target=dirwatch.watch_directories, args=([watchdir],self.OnFilesChanged,1) ).start()

    def OnAddImages(self,event=None):
        try:
            dialog = wx.FileDialog(self,style=wx.MULTIPLE)
            ret = dialog.ShowModal()
            if ret == wx.ID_OK:
                filenames = dialog.GetPaths()
                dialog.Destroy()
                for f in filenames:
                    wx.CallAfter(self.OpenImage,f)

                    # upload the file to venue
                    self.dataStoreClient.Upload(f)

                    # send event notifying others to upload
                    filename = os.path.basename(f)
                    self.sharedAppClient.SendEvent(SharedImageViewer.LOAD_IMAGE,filename)

                # update the application state
                keys= self.sharedAppClient.GetDataKeys()
                filelist = map( lambda x: os.path.basename(x), filenames)
                if SharedImageViewer.imagelist not in keys:
                    self.sharedAppClient.SetData(SharedImageViewer.imagelist,filelist)
                else:
                    myfileliststr = self.sharedAppClient.GetData(SharedImageViewer.imagelist)
                    myfilelist = eval(myfileliststr)
                    myfilelist += filelist
                    self.sharedAppClient.SetData(SharedImageViewer.imagelist,myfilelist)
                    
        except:
            import traceback
            traceback.print_exc()
            self.log.exception('Error in OnLoad')


    def OnImageSelect(self,event):
        try:
            o = event.GetEventObject()
            image = self.buttonImageMapping[o]
            self.ShowImage(image)
            name = None
            for k,v in self.nameImageMapping.items():
                if v == image:
                    name = k
            if name:
                self.sharedAppClient.SendEvent(SharedImageViewer.SHOW_IMAGE,name)
            else:
                print 'Image not found in mapping'
        except:
            import traceback
            traceback.print_exc()
            self.log.exception('Error in OnImageSelect')

    def OnClose(self,event):
        self.Shutdown()
        event.Skip()


if __name__ == '__main__':

    import sys
    import wx
    from twisted.internet import reactor
    import getopt

    try:
        opts, args = getopt.getopt(sys.argv[1:], "a:c:v:",
                                   ["applicationURL=",
                                    "connectionId="])
    except getopt.GetoptError,e:
        #Usage()
        print e
        sys.exit(2)

    appURL = None
    connectionId = None
    for o, a in opts:
        if o in ("-a", "--applicationURL"):
            appURL = a
        elif o in ("-c", "--connectionId"):
            connectionId = a
        elif o in ("-v"):
            pass

    sys.argv[1:] = []
    agapp = Application.instance()
    agapp.Initialize()

    try:
        from twisted.internet import _threadedselect as threadedselectreactor
    except:
        from twisted.internet import threadedselectreactor

    wxapp = wx.PySimpleApp()
    wx.InitAllImageHandlers()
    sharedImageViewer = SharedImageViewer(None,-1,'Shared Image Viewer', appURL, connectionId, size=(600,500))
    sharedImageViewer.Show(1)
    sharedImageViewer.DoLateJoin()
    sharedImageViewer.Refresh()
    sharedImageViewer.sizer.Layout()

    reactor.interleave(wx.CallAfter)

    wxapp.MainLoop()

    sharedImageViewer.Shutdown()


